close all
clear all
clc
set(0,'DefaultFigureColormap',feval('gray')); % Change the default colormap to gray

%% Before doing Image analysis, let's lear how to manipulate images. 

%% What is an image?
% A grayscale image is a 2D matrix.
% A colorscale image is a 3D matrix of third dimension 3 (RGB channels)
Objects
% in Object I defined three images I1, I2, I3. To visualise them you can
% use the function images:
figure(1)
imagesc(I1);
% Check the other two images. 
% to get the size of an image you can use the function size:
size(I1)
% Check the size of each image, and confirm whether they are greyscale or
% RGB images. 

%% Images and memory alocation
% Each image or matrix has a type, which correspond to the memory (numer of bytes) allocated to
% each pixel. For instance in a uint8 image, a pixel value goes from 0 to
% 255, with only integer. This limit the precision of a picel. Try: 
A=uint8([[1 2 3];[4 5 6];[7 8 9]]);
disp(A/2)
% In matlab the default type in matlab is called double. Double are great to store large value with high
% precision, because they store them as scientific notation i.e.
% a=2.56*10^32. 
% Matrices created with matlab are double by default. Images imported can
% have other type; which is why I recommand to convert them to double as
% soon as they ar imported. Ex:
I1=imread('coins.png');
I2=double(I1);
figure(1)
subplot(2,1,1)
imagesc(I1/100)
subplot(2,1,2)
imagesc(I2/100)

%% Basic matrices creation and manipulation
% Vectors are special matrices of dimension 1; they are made using [] or :
% for regularly-spaced vectors (which is EXTREMELY useful). 
v1=[0 1 0 1 2 1];
v2=0:50;
v3=-1:0.1:1;
v4=[-1 0:0.1:1 2];
% A vector column can be created by taking the transpose of a vector:
v5=v1';
% Nice functions for vectors: 
help max
help min
help sort
help sum
% A 2D matrix is defined by []; inside a line numbers are separated by a
% space or a , (I recomand the , which is more visible and thus avoid
% errors). Different lines are separated by ;
A=[1,2,3;4,5,6;7,8,9];
% Similarly, Try to make a matrix B of size 3x3 where all elements are 0
% except for the center that is 1. 
% Large matrices are defined by a function, or by a loop. It can be usefull
% to create a matrix of zeros:
A2=zeros(4,2);
% Or a matrix of 'NaN' (NaN stands for Not A Number)
A3=NaN(4,2);
% Or a matrix of ones
A4=ones(4,2);
% You can also create matrices by concatenating vectors:
A5=[v2;v2;v2];

%% Basic Matrix indexing
% Once made, element of a matrice can be reached thanks to its indexing. 
% There is two ways of indexing a matrice:
%   1/ the one number indexing: each element is indexed by
%   a number, which increase from left to right and then from top to
%   bottom. 
A(3)
% Find which index you need to reach the 8 in A
% Find which index you need to reach the 1 in B
%   2/ the dimensional indexing, where N is the dimension of the matrix: each
%   element is indexed by N subscripts which correspond to its coordinate
%   inside the matrix. ex for a matrix of dimension 2 (like A or B), an
%   element is indexed by it's row and column subscripts:
A(1,2)
% Find which index you need to reach the 8 in A
% Find which index you need to reach the 1 in B
% The dimensional indexing is usualy more useful, as more practical. If the size of the
% matrice is known, then one can pass from the one number indexing to the
% dimensional indexing, and reciprocally: see what the functions sub2ind/ind2sub do. 
% It is also possible to reach elements with an indexing vector:
A(1:3,1)
A(2,1:3)
% And the indexing word end is also usefull, it give back the last element
% of a matrix:
v=1:50;
v(end)
v(end-2)
v(end-5:end)
% Finaly, if you want to look at every thin in one dimension:
A(:,1) % is totally equivalent to A(1:end,1). 
% More about indexing: https://www.mathworks.com/company/newsletters/articles/matrix-indexing-in-matlab.html

%% Elaborate Matrix indexing
% We saw that matrices could be indexed by vectors. This allows very fancy
% manipulation of matrices and images. For instance given the image:
A=[[2 1 2];[0 1 0];[2 1 2]];
figure(1)
clf %clear th current figure
subplot(2,1,1)
imagesc(A)
% I am going to create a vector v that contains the index of all 1 pixels (white
% pixels) of the image:
v=[4 5 6];
%Then I can call an manipulate these pixels independently of all others.
A(v) % simply calling them. Check that it's indeed all the  pixel equal to one.
A(v)=0; % Changing them to 0, now the image has changed: 
subplot(2,1,2)
imagesc(A)
% This time I designed v manually, but it can be done automatically.

%% Designing an indexing vector
% An indexing vector can be designed using logical conditions. Typical
% logical condtions: > < or ==. The indexes are found:
% 1/ using the find function:
A=[[1 2 3];[4 5 6];[7 8 9]];
figure(1)
clf
subplot(3,1,1)
imagesc(A)
% Finding the indexes of the pixels smaller than 5:
v=find(A<5);
% Check 
A(v) 
% They should be all smaller than 5;
% And then I can change them, putting them to 1 for instance:
A(v)=1;
subplot(3,1,2)
imagesc(A)
% 2/ By direct indexing: It's exactly the same process, but condensed in
% one line:
A=[[1 2 3];[4 5 6];[7 8 9]];
A(A<5)=1;  % this line replace the lines v=find(A<5); and A(v)=1;
subplot(3,1,3)
imagesc(A)

%% Or | and And &
% Or | and And & allows to combine multiple conditions. 
% Or | and And & are logical operations; if c1 and c2 are conditions; 
c1=1; % c1 is true
c2=0; % c2 is false
c=c1 & c2 % is true only if c1 AND c2 are true. 
c=c1 | c2 % is true only if c1 OR c2 are true. 
% Try the following example:
v1=[ 1 1 0 0];
v2=[1 0 1 0];
% Try to predict before computing it what v1 & v2 and v1|v2 will do. 

%% Exercise
Im=double(imread('mandi.tif'));
% Try to change the pixels higher than 60% or smaller than 30% of the
% maximum pixel intensity to zero. 

% Successive tip (start by thinking without looking at them)
% 1/ find the maximum intensity M of the image using mas
% 2/ find all pixels greater than 0.8*M OR smaller than 0.2*M
% 3/ Using that indexing vector, turn these pixels to 0. 
% Image the initial and final images. 



