close all
clear all
clc

%% An object in an image
% Segmenting an object from an image means getting all the pixel
% constituting the object. The following fucntion shows you a typical
% example of segmentation, where the segmented object is in white:
BasicSegmentation
% To segment an object, the main and most important point is to pinpoint
% what make the object different from the background. The more different
% from the background the object is; the easiest it will be to segmente. 
% The most basic characteristic of an object one can use are its intensity,
% its size, its position and its shape. An illustration:
TypeOfSegmentation
% In a real problem, usually one use a combination of these
% characteristics. 

%% Basic segmentation - The intensity
% Create the following image
Im=GreyGradCircles(1,1);
figure(1)
imagesc(Im)
colormap gray
axis equal
% How to get all pixels from the white circle? Check the value of the white
% pixel, then use the function find to get the value of these pixels. 

% You have segmented the white circle in the image. 
% Now create the following image:
Im=GreyGradCircles(2,1);
figure(1)
imagesc(Im)
colormap gray
axis equal
% Like before, try to segment the brighter circle only, the darker circle
% only, and the two circle. 
% To make an image from the segmented object, You can create a matrix of
% zeros of the same size as the initial image, 
Ires=zeros(size(Im));
% and then attribute the value 1 at the segmented pixel. If pix is the name
% of the pixel, Ires(pix)=1; Then you can image the resulting image. 

% Aplication exercise to try later: 
% Try to segment different circle in the following image: 
Im=GreyGradCircles(5,4);
figure(1)
imagesc(Im)
colormap gray
axis equal

% More generally, the object one want to segment might not have a constant
% intensity. Generate the folowing image: 
Im=GreyRandCircles(2,2,2);
figure(1)
subplot(1,2,1)
imagesc(Im)
colormap gray
axis equal
% While you can say that one circle is brighter than the others, it does
% not have a constant intensity, so you cannot segmente it by looking at
% all the pixels of intensity equal to a certain value. In that case, one
% use a threshold: an intensity value above which you look for the pixels.
% You could implement it manually by using find and a > condition;
% pix=find(Im>t), where t is the threshold intensity; and then create a
% black and white images from the pixels. Matlab has conveniently an
% inbuilt function for that, that directly creates a binary image:
t=140;
I2=imbinarize(Im,t);
subplot(1,2,2)
imagesc(I2)
colormap gray
axis equal
% Play with t in order to segment only the brightest circle.  

%% Geometrical properties of an object 
% The geometrical properties of an object can be extracted from the pixel
% list. i.e. the center of an object is the mean of the pixel position; its
% radius, major axis lenght and minor axis lenght can be computed
% respectively by computing as the average, maximum and minimum distance
% between the edge pixels and the center, etc... Each of these requires a
% matlab code, you can try to build them yourself as an exercise. Once
% again fortunatly, Matlab as an inbuildt function that does everything:
% regionprops. This regionprops function takes as argument a binary image,
% and the properties youwant to measure; and give back a structure with the
% measured properties. An example: 
Im=WeirdObj;
prop=regionprops(Im,'Centroid','EquivDiameter','MajorAxisLength','MinorAxisLength','orientation');
C=prop.Centroid;
R=prop.EquivDiameter/2;
a1=prop.MajorAxisLength/2;
a2=prop.MinorAxisLength/2;
alpha=prop.Orientation;
% Plotting the measured properties:
figure(1)
imagesc(Im)
axis equal
colormap gray
hold on
plot(C(1),C(2),'k+')
t=0:0.1:2*pi;
plot(C(1)+R*cos(t),C(2)+R*sin(t),'r')
plot([C(1) C(1)+a1*cosd(alpha)],[C(2) C(2)-a1*sind(alpha)],'g')
plot([C(1) C(1)-a2*sind(alpha)],[C(2) C(2)-a2*cosd(alpha)],'b')
legend('Center','Equivalent circle','half major axis','half minor axis')
hold off
% To get a list of all the properties regionprops can measure:
help regionprops

%% Segmenting multiple objects using geometrical properties
% To segment an object using geometrical properties among multiple objects;
% one need to be able to get the pixel of each object separately. 
% The most common way to do so is to use the connectivity of object : the
% function bwconncomp detect all disconnected object within a binary image.
Im=BinaryObjects;
figure(1)
subplot(1,2,1)
imagesc(Im);
colormap gray
axis equal;
E=bwconncomp(Im);
% Inside E there is the connectivity used to separate the objects, the
% imagesize, the number of objects and the Pixel List forming each object.
% You can now use these pixel list to extract the geeometrical properties
% of the object; and select the object according to your need.
prop=regionprops(E,'centroid');
C=cat(1,prop.Centroid); % C now contains all the center of the objects of the image:
figure(1)
subplot(1,2,1)
hold on
plot(C(:,1),C(:,2),'+r')
hold off
% You can now selectively select objects according to their geometrical
% properties. For instance, let's keep only the object of aspect ratio 1:
prop=regionprops(E,'MajorAxisLength','MinorAxisLength');
a1=cat(1,prop.MajorAxisLength);
a2=cat(1,prop.MinorAxisLength);
ar=a1./a2; % ar is then the aspect ratio of each object in the image. 
I2=zeros(size(Im));
pix=cat(1,E.PixelIdxList{(ar==1)});
I2(pix) = 1;
subplot(1,2,2)
imagesc(I2)
colormap gray
axis equal
% Similarly, try to draw all objects whose centers are in the upper half of
% the image. 

%% Aplication exercise - Detecting a jumping bubble. 
% Together:
%1/ Open the avi file JumpingDroplet
%2/ Binarize the first image
%3/ Get the position and radius of all objects
%4/ Segment the bubble for the first image. 
%5/ Make a loop on each image of the movie.
%6/ For each image, segment the bubble, get its position and radius. 
%7/ Plot the evolution of the droplet position and radius with time. 

% On your own:
%1/ Open the avi file JumpingDroplet2
%2/ Binarize the first image
%3/ Get the position and radius of all objects
%4/ Segment the bubble for the first image. 
%5/ Make a loop on each image of the movie.
%6/ For each image, segment the bubble, get its position and radius. 
%7/ Plot the evolution of the droplet position and radius with time. 




