function [] = VariousFilters()
% Generate Various example of filters

%% Example image
I = imread('cameraman.tif');
figure(1)
subplot(3,3,1)
imagesc(I)
colormap gray
axis equal
title('Original image')

%% Kernel convolution
% Gaussian filter
% convolute a square Gaussian kernel. 
% Blure the i;age - useful to get background intensity gradient (and then remove it from image).
I1=imgaussfilt(I,5);
subplot(3,3,2)
imagesc(I1)
axis equal
title('Gaussian filter')
% Median filter
% convolute a Median filter
% Remove small area - useful to remove some irregularities in an image. 
I2=medfilt2(I,[10 10]);
subplot(3,3,3)
imagesc(I2);
axis equal
title('Median Filter')

%% Example image
J=imread('coins.png');
J0=imbinarize(J,0.7);
subplot(3,3,4)
imagesc(J0)
axis equal
title('Original image')

%% Morphological operations
% Dilatation
% Dilate the image by using an object
se=strel('disk',5);
J1=imdilate(J0,se);
subplot(3,3,5)
imagesc(J1)
axis equal
title('Dilatation with a disk')
% Closing
% Successive ersosion and dilatation filters
% Close small holes
J2=imclose(J0,se);
subplot(3,3,6)
imagesc(J2)
axis equal
title('Closing using a disk')

%% Example image
K=rgb2gray(imread('tissue.png'));
K=K(300:400,200:400);
subplot(3,3,7)
imagesc(K)
axis equal
title('Original image')

%% Frequency filter
% Wiener filter
% Adaptive low-pass Wiener filter
% Useful to remove noise
K1=wiener2(K,0.01);
subplot(3,3,8)
imagesc(K1)
axis equal
title('Wiener Filter (Low pass)')
% High-pass filter
% Use the fourier transform (fft) to filter the frequency
nx = size(K, 2);
hx = ceil(nx/2)-1;
ftdiff = (2i*pi/nx)*(0:hx);     % ik 
ftdiff(nx:-1:nx-hx+1) = -ftdiff(2:hx+1);  % correct conjugate symmetry
% compute "gradient" in x using fft
K2 = ifft2( bsxfun(@times, fft2(K), ftdiff) );
subplot(3,3,9)
imagesc(K2)
axis equal
title('High-pass filter using fourier transform')

end

