close all
clear all
clc

%% From a raw image to a ready to be binarized image.
% In Part 1, we saw how to segment objects by binarizing an image. The
% images we used were "ready to be binarized", meaning that a simple
% intensity threshold was enough to differenciate the object from the
% background. Real images may have a complex background, and a complex
% object. Today, we will see learn how to use various filters and how to
% remove a background. 

%% Various kind of filters
% This part is non exhaustive, and based on what I use the most. 
VariousFilters 
% I use mostly three kind of filters in matlab, Kernel convolution,
% Morphological tools, Frequency filters. 
% 1/ Kernel convolution
% Convolute a filter
% ex : imgaussfilt, Convolute a square Gaussian kernel. Blure the image
%      -> useful to get background intensity gradient (and then remove it
%         from image).
%      medfilt2, Convolute a Median filter. Remove small area
%      -> useful to remove some irregularities in an image.
%       
% 2/ Morphological operations
% Analysis and processing of geometrical structures.
% i.e. Use the shape of images
% ex : imdilate, dilate the image by using an object
%      imerode, erode the image using an object
%      imopen, succesive erosion and dilatation, open holes.
%      -> useful to erase very small objects.
%      imclose, succesive dilatation and erosion, close holes.
%      -> useful to close small all inside an object.
%
% 2/ Frequency filter
% Filter spacial frequency in an image, usually using Fourier transform. 
% ex: wiener2, adaptive low-pass Wiener filter
%     -> useful to denoise an image. 

%% Aplication exercise: 
% I0 is the original image. I1 to I3 are images on which have been used an
% unknown filters. Try to find what filter has been used by playing with
% the different filters introduced before. Use help to understand how to
% use these filters.
[I0,I1,I2,I3]=MysteriousFilter;
figure(1)
subplot(2,2,1)
imagesc(I0)
title('Original')
subplot(2,2,2)
imagesc(I1)
title('Filter 1')
subplot(2,2,3)
imagesc(I2)
title('Filter 2')
subplot(2,2,4)
imagesc(I3)
title('Filter 3')

%% Removing a background
% When you have the background of you image, it helps a lot with the later
% segmentation of the objects. 
% In the Part2 folder, there is two movies, ShrinkingDroplet and
% FlowingGrains, and their two corresponding backgrounds, BackSD and
% BackFG. Create a function named BackSub whose input are the movie name
% and background name,  purpose is to open the movies and the background,
% substract the background for each image of the movie, and display side by
% side the initial and resulting image. 
% Note that I created a useful function, VideoOpener, that reads an avi
% file and give back directly a 3D matrix (just like what we did last
% time).

%% Computing a background
% Sometimes there is no available background, and it needs to be computed
% from theraw images. There is several ways to do that, depending on the
% kind of background you want to extract. The basic idea is to use the
% properties of the object you want to segment to make it "dispear". For
% example:
% 1/ If the object is small
% A filter that delete small objects can allow to compute a background. Ex,
% a gaussian filter or a opening filter.
% 2/ If the object is moving
% An average of the images over time will give the background. 
% From my personal experience, 1/ is useful to remove heterogeneity in
% intensity of a background, and 2/ is useful for an object that moves a
% lot in the image. 

%% Aplication example: 
% Compute the background for the following two examples: the Pillar.png
% image, and the movie CollidingDroplets.avi. 

%% Summary
% The standard procedure for image processing: 
% 1/ Import image and convert to double
% 2/ Compute/substract background
% 3/ Apply filters : ex noise filters or morphological filters. 
% 4/ Binarize
% 5/ Segment object
% 6/ Discriminate object using their geometrical properties.
% 7/ Get the variable of interest versus time (ex: the position/radius). 

%% Application exercises
% Ex1: CollidingDroplets.avi -> Get the evolution of the distance between
%       droplet with time. 
% Ex2: ShrinkingDroplet.avi -> Get the evolution of the droplet radius with
%       time. 
%       hints: to remove the reflexion, you might want to manualy define
%       the interface. 
% Ex3: FlowingGrains.avi -> Get the average velocity of the grains. 
%       hints: the number of grains varies, you might want to use a cell to
%       store the position with time. 







